# -*- coding: iso-8859-1 -*-

__doc__ = """ 
This routine compares (precomputed) time series of model fields

Input: numpy files with time series
Output: plots
"""

__author__ = "Emanuele Emili - April 2012"

import os
os.environ['QT_QPA_PLATFORM']='offscreen'
import numpy as np
import seaborn as sns
from matplotlib import pylab as plt
from matplotlib.ticker import AutoMinorLocator, MaxNLocator
# from taylorDiagram import TaylorDiagram
import glob
from datetime import datetime, timedelta
import subprocess

def PlotBars (fig,xpos,x,dictionary):
    width = dictionary['width']
    plt.title(dictionary['title'])
    plt.ylabel(dictionary['ylabel'])
    #print xpos, x
    plt.bar(xpos,x,width,color=plotformat['color'],label=dictionary['label'])
    # add some
    ax = fig.gca()
    ax.set_ylabel(dictionary['ylabel'])
    if len(filelist) == 2: ax.set_xticks([0.2,0.8])
    if len(filelist) == 3: ax.set_xticks([0.3,1.1])
    ax.set_xticklabels(dictionary['xtickslabels'])
    ax.set_ylim(dictionary['yrange'][0],dictionary['yrange'][1])
    ax.set_xlim(dictionary['xrange'][0],dictionary['xrange'][1])

def PlotTaylor(fig,data,plotcfg,subplotn):
    
    stdrefs = data['stdo'][0]
    samples = [[triade[0],triade[1],triade[2]] for triade in zip(data['stdm'],data['corr'],plotcfg['labels'])]
    colors = plt.matplotlib.cm.Dark2(numpy.linspace(0,1,len(data['stdm'])))
    markers = ["o","v","D","p","P","s"]
    
    #plt.grid(False,linestyle='--')
    dia = TaylorDiagram(stdrefs, fig=fig, rect=subplotn,
                        label='Reference',srange=(0.2, 2.2))
    #dia._ax.grid(True,linestyle='--')
    # Add samples to Taylor diagram
    for i,(stddev,corrcoef,name) in enumerate(samples):
        dia.add_sample(stddev, corrcoef,
                       marker=markers[i], ms=10, ls='',
                       #mfc='k', mec='k', # B&W
                       mfc=colors[i], mec=colors[i], # Colors
                       label=name,zorder=10+i)
        
    # Add RMS contours, and label them
    contours = dia.add_contours(levels=5, colors='0.5') # 5 levels
    dia.ax.clabel(contours, inline=1, fontsize=10, fmt='%.1f')
    # Tricky: ax is the polar ax (used for plots), _ax is the
    # container (used for layout)
    if 'title' in plotcfg: dia._ax.set_title(plotcfg['title'])
    
    return dia

def ReadTimeSeries(dirlist,plotformat):
    
    combine_plots = True
    slev = 23
    
    for icol,col in enumerate(columns):
        
        title = ''
        fig = plt.figure(figsize=figsize,dpi=200)
        plt.subplots_adjust(hspace=0.1)

        # plt.subplot(1,2,1)
        plt.subplot(1,1,1)
        fig.autofmt_xdate()
        ax1 = fig.gca()
        ax1.set_xlabel('Date')
        ax1.set_title(col+' avg')

        # plt.subplot(1,2,2)
        # fig.autofmt_xdate()
        # ax2 = fig.gca()
        # ax2.set_xlabel('Date')
        # ax2.set_title(col+' std')
        # Find the appropriate file
        for nf,fdir in enumerate(dirlist):
            for ic in range(1,ncycl+1):

                fdir = fdir.replace('cycle_0001','cycle_%04i' % (ic))
                ldate = '%s_%s' % ((datetime.strptime(dates[nf].split('_')[0],'%Y%m%d%H')+dtcycl*(ic-1)).strftime('%Y%m%d%H'),\
                        (datetime.strptime(dates[nf].split('_')[1],'%Y%m%d%H')+dtcycl*(ic-1)).strftime('%Y%m%d%H'))
                filelist = sorted(glob.glob(\
                                os.path.join(fdir,'fields_plots',
                                             "*_%s_lev_%02i_%s_timeseries.npz" % (col.replace('.','_'),slev,ldate))))
                if len(filelist) != 1 :
                    print("Anomaly: did not find a unique filename for %s %s in %s" % (col,ldate,fdir))
                    pass
                else:
                    lfile = filelist[0]
                    data = np.load(lfile,allow_pickle=True)
                
                    avg = np.ma.masked_values(data['landavg'],data['missing_value'])
                    std = np.ma.masked_values(data['landstd'],data['missing_value'])

                    title = title + linenames[nf]+ '= %4.2e ' % (np.ma.mean(avg))
                    # ax1.plot(data['time'],avg,label=linenames[nf]+' (%4.2e)' % (np.ma.mean(avg)))
                    ax1.plot(data['time'],avg,label=linenames[nf])
                    # ax2.plot(data['time'],std,label=linenames[nf]+' (%4.2e)' % (np.ma.mean(std)))
        
        # ax1.set_title(col+' mean: ' + title)
        if icol == 0: ax1.legend(loc='upper right')
        # ax2.legend()
        plt.savefig(os.path.join(outputdir,fname+'_'+col.replace('.','_')))
        plt.close()

    # Combine all maps in a single page if needed
    if combine_plots:

        combine_cmd = 'montage -mode concatenate -tile 3x4 {'

        for isp,spc in enumerate(columns):
            if isp > 11: break # limited to 12 species for the moment
            combine_cmd = combine_cmd + os.path.join(outputdir,fname+'_'+spc.replace('.','_')+'.png')
            if isp < len(columns)-1: combine_cmd=combine_cmd+','

        combine_cmd=combine_cmd+'} '+os.path.join(outputdir,fname+'_allspc.png')
        print(combine_cmd)
        subprocess.run(combine_cmd, shell=True)
    
if __name__ == "__main__":
    
    sns.set() # Apply seaborn beautificator
    
    # OSSE
    dirlist = []
    # dirlist.append('/scratch/ae/emili/seeds/osse/exp_30/truth/FORECAST')
    # dirlist.append('/scratch/ae/emili/seeds/osse/exp_30/perturbed_forecast/FORECAST/fields_diff/truth')
    # dirlist.append('/scratch/ae/emili/seeds/osse/exp_58/envar')
    # dirlist.append('/scratch/ae/emili/seeds/osse/exp_55/envar')
    # dirlist.append('/scratch/ae/emili/seeds/osse/exp_56/envar')
    # dirlist.append('/scratch/ae/emili/seeds/osse/exp_47/envar')
    # dirlist.append('/scratch/ae/emili/seeds/osse/exp_31/model_error_forecast/FORECAST')

    # dirlist.append('/scratch/ae/emili/seeds/osse/exp_31/final_forecast/FORECAST')
    # dirlist.append('/scratch/ae/emili/seeds/osse/exp_31/final_forecast/FORECAST/bias_correction')
    # dirlist.append('/scratch/ae/emili/seeds/osse/exp_31/envar')
    # dirlist.append('/scratch/ae/emili/seeds/osse/exp_31/final_forecast/FORECAST')
    # dirlist.append('/scratch/ae/emili/seeds/osse/exp_31/final_forecast/FORECAST/bias_correction')

    # dirlist.append('/scratch/ae/emili/seeds/osse/exp_30/perturbed_forecast/FORECAST/fields_diff/truth')
    # dirlist.append('/scratch/ae/emili/seeds/osse/exp_35/envar/fields_diff/truth')
    # dirlist.append('/scratch/ae/emili/seeds/osse/exp_36/envar/fields_diff/truth')
    # dirlist.append('/scratch/ae/emili/seeds/osse/exp_30/final_forecast/FORECAST/fields_diff/truth')
    # dirlist.append('/scratch/ae/emili/seeds/osse/exp_30/final_forecast/FORECAST/bias_correction/fields_diff/truth')
    # dirlist.append('/scratch/ae/emili/seeds/osse/exp_37/envar/fields_diff/truth')
    # dirlist.append('/scratch/ae/emili/seeds/osse/exp_31/final_forecast/FORECAST/fields_diff/truth')
    # dirlist.append('/scratch/ae/emili/seeds/osse/exp_31/final_forecast/FORECAST/bias_correction/fields_diff/truth')

    # dirlist.append('/scratch/ae/emili/seeds/simu/exp_00/forecast/FORECAST')
    # dirlist.append('/scratch/ae/emili/seeds/simu/exp_53/envar')
    # dirlist.append('/scratch/ae/emili/seeds/simu/exp_61/envar')
    # dirlist.append('/scratch/ae/emili/seeds/simu/exp_62/envar')
    # dirlist.append('/scratch/ae/emili/seeds/simu/exp_63/envar')
    # dirlist.append('/scratch/ae/emili/seeds/simu/exp_61/envar')

    # dirlist.append('/scratch/ae/emili/seeds/long/exp_00/cycle_0001/forecast/FORECAST')
    # dirlist.append('/scratch/ae/emili/seeds/long/exp_00/cycle_0001/envar')
    # dirlist.append('/scratch/ae/emili/seeds/long/exp_00/cycle_0001/final_forecast/FORECAST')
    # dirlist.append('/scratch/ae/emili/seeds/long/exp_00/cycle_0001/final_forecast/FORECAST/bias_correction')

    # dirlist.append('/scratch/globc/emili/seeds/osse/exp_003/cycle_0001/perturbed_forecast/FORECAST/fields_diff/truth')
    # dirlist.append('/scratch/globc/emili/seeds/osse/exp_003/cycle_0001/envar/fields_diff/truth')
    # dirlist.append('/scratch/globc/emili/seeds/osse/exp_003/cycle_0001/final_forecast/FORECAST/fields_diff/truth')
    # dirlist.append('/scratch/globc/emili/seeds/osse/exp_003/cycle_0001/final_forecast/FORECAST/bias_correction/fields_diff/truth')

    # For the report
    # dirlist.append('/scratch/globc/emili/seeds/osse/exp_005/cycle_0001/model_error_truth/FORECAST/fields_diff/truth')
    # dirlist.append('/scratch/globc/emili/seeds/osse/exp_005/cycle_0001/model_error_forecast/FORECAST/fields_diff/envar')

    # dirlist.append('/scratch/globc/emili/seeds/osse/exp_005/cycle_0001/truth/FORECAST')
    # dirlist.append('/scratch/globc/emili/seeds/osse/exp_005/cycle_0001/perturbed_forecast/FORECAST')
    # dirlist.append('/scratch/globc/emili/seeds/osse/exp_005/cycle_0001/envar')
    # dirlist.append('/scratch/globc/emili/seeds/osse/exp_005/cycle_0001/final_forecast/FORECAST')
    # dirlist.append('/scratch/globc/emili/seeds/osse/exp_005/cycle_0001/final_forecast/FORECAST/bias_correction')

    dirlist.append('/esarchive/scratch/eemili/results/aq-monarch/initest_001/cycle_0001/forecast')
    dirlist.append('/esarchive/scratch/eemili/results/aq-monarch/initest_001/cycle_0001/envar')
    #dirlist.append('/esarchive/scratch/eemili/results/aq-monarch/initest_001/cycle_0001/letkf')
    #dirlist.append('/esarchive/scratch/eemili/results/aq-monarch/initest_002/cycle_0001/envar')
    #dirlist.append('/esarchive/scratch/eemili/results/aq-monarch/initest_003/cycle_0001/envar')
    dirlist.append('/esarchive/scratch/eemili/results/aq-monarch/initest_005/cycle_0001/envar')

    dates = []
    # dates.append('2010070400_2010070700')
    # dates.append('2010070400_2010070700')
    # dates.append('2010070400_2010070500')
    # dates.append('2010070400_2010070500')
    # dates.append('2010070400_2010070500')
    # dates.append('2010070400_2010070500')
    # dates.append('2010070400_2010070500')
    # dates.append('2010070400_2010070500')
    # dates.append('2010070500_2010070700')
    # dates.append('2010070500_2010070700')
    # dates.append('2010070400_2010070500')
    # dates.append('2010070500_2010070700')
    # dates.append('2010070500_2010070700')
    # dates.append('2019061900_2019062200')
    # dates.append('2019061900_2019062200')
    # dates.append('2019061900_2019062200')
    # dates.append('20190619T00_20190619T23')
    # dates.append('20190619T00_20190619T23')
    # dates.append('2019061900_2019062000')
    # dates.append('2019061900_2019062000')
    # dates.append('20190619T00_20190619T23')
    # dates.append('20190619T00_20190630T23')
    # dates.append('20190619T00_20190619T23')
    # dates.append('2019062000_2019062200')
    # dates.append('2019062000_2019062200')
    # dates.append('20190620T00_20190621T23')

    #dates.append('2023100300_2023100400')
    dates.append('2023100300_2023100400')
    dates.append('2023100300_2023100400')
    #dates.append('2023100300_2023100400')
    #dates.append('2023100300_2023100400')
    dates.append('2023100300_2023100400')

    linenames = []
    linenames.append('forecast')
    linenames.append('4denvar_001')
    #linenames.append('letkf')
    linenames.append('4denvar_005')
    #linenames.append('4denvar_003')
    #linenames.append('4denvar_004')
    # linenames.append('true_model_error')
    # linenames.append('estimated_model_error')
    # linenames.append('truemodelerror_30')
    # linenames.append('final_fct')
    # linenames.append('final_fct_corr')
    # linenames.append('envar_31')
    # linenames.append('finalfct_31')
    # linenames.append('finalfctbcorr_31')

    columns = []
    # columns.append('O_x')
    # columns.append('NO_2')
    # columns.append('CO')
    # columns.append('NO')
    # columns.append('NH_3')
    # columns.append('HNO_3')
    # columns.append('CH_4')
    # columns.append('HCHO')
    # columns.append('HO_2')
    # columns.append('H_2O_2')
    # columns.append('H_2O')
    # columns.append('CH_3')
    # columns.append('CH_3O')
    # columns.append('N_2O')
    # columns.append('N_2O_5')
    # columns.append('ISO')
    # columns.append('SO_2')
    # columns.append('PANs')
    # columns.append('NMVOCs')
    # columns.append('PM10')
    # columns.append('PM2.5')
    columns.append('O3')
    columns.append('NO2')
    columns.append('NO')
    columns.append('CO')
    columns.append('SO2')
    columns.append('NH3')
 
    ncycl = 1
    dtcycl = timedelta(days=1)

    # outputdir = "/scratch/ae/emili/seeds/long/exp_00/timeseries"
    outputdir = "/esarchive/scratch/eemili/results/aq-monarch/initest_001/comparisons/fields"

    if not os.path.exists(outputdir) : os.makedirs(outputdir)

    os.chdir(outputdir)
    
    domain = "MACC01"
    
    expname = ''
    
    # figsize = (15,6)
    figsize = (8,6)

    outfilenames = [string for string in linenames]

    fname = outfilenames[0]
    for string in outfilenames[1:]: fname = fname+'_'+string 
    if expname != '': fname = fname + '_' + expname
    
    plotformat = {}
    ReadTimeSeries(dirlist,plotformat)
        
