# -*- coding: utf-8 -*-
"""
Class handling the various ecflow steps for assimilation experiments 
"""

import os
# from ruamel.yaml import YAML
import yaml
from yaml import CLoader as Loader, CDumper as Dumper
import isodate.isodatetime as isodatetime
import isodate.isoduration as isodur
from datetime import timedelta
from Jedi_task import Jedi_task, ListMocageOutput, GetForecast, PrepareJob
import itertools
import math
from aq_steps import machines_config

class Localization(Jedi_task):

  def __init__(self, **kwargs):

    super().__init__(**kwargs)

  @classmethod
  def py_setup_script(cls):

    kwargs = cls.load_ecfvars()

    # Load the dirac namelist here (my version, not the complete oops one)
    with open(kwargs['namelist']) as file:
        cfg = yaml.load(file,Loader=Loader)
    # Remember that expname needs to come through ecflow because this method does not know self
    # and does not use the global experiment namelist! I can come back on this decision if I find 
    # out that a global namelist avoids a lot of duplication (e.g. output ensemble -> input dirac 
    # must be specified twice). In that case remove all the static ecflow arguments in favor
    # of the namelist usage.

    outdir = kwargs['outpath']

    if not os.path.exists(outdir): os.makedirs(outdir)

    fctname, bkgdir = GetForecast(cfg,kwargs)

    print('>> Initializing BUMP preprocessor: namelist creation...')

    outyaml = {'background':{}}

    statevars = cfg["state variables"]
    outyaml['input variables'] = statevars

    outyaml['geometry'] = cfg['grid']

    outyaml['background'] = ListMocageOutput(bkgdir,[isodatetime.parse_datetime(kwargs['dateini'])],statevars,outyaml['geometry']['domname'])

    outyaml['bump'] = {
        'io':{
            'data directory': outdir,
            'files prefix': 'loc_1pe'
        },        
        'drivers':{
            'multivariate strategy':cfg['multivariate'],
            'compute nicas': True,
            'write local nicas' : True
        },
        'nicas':{
            'resolution': cfg['resol'],
            'explicit length-scales': True,
            'horizontal length-scale': [{'groups':['common'],'value':cfg['rh']}],
            'vertical length-scale':[{'groups':['common'],'value':cfg['rv']}],
            'common localization weights': [{'row variables':statevars,'column variables':statevars,'value':cfg['mc']}]
        }
        }

    # Older way of specifing multivariate localization
    # if cfg['multivariate'] == 'common_weighted':
    #     # Construct the multivariate localization
    #     pair_order_list = list(itertools.combinations(statevars,2))
    #     outyaml['bump']['loc_wgt'] = {el[0]+'-'+el[1]:0.5 for el in pair_order_list}

    print(yaml.dump(outyaml))

    with open(kwargs['jedinamelist'],'w') as f:
        print(yaml.dump(outyaml),file=f)

    nodes = 1
    queue = 'interactive'
    mpi_per_slot = 1
    hostname = 'kglobc1'

    if 'computing' in cfg and 'run' in cfg['computing']:
      hostname = cfg['computing']['run']['machine']
      if 'nodes' in cfg['computing']['run']: nodes = cfg['computing']['run']['nodes']
      if 'queue' in cfg['computing']['run']: queue = cfg['computing']['run']['queue']
      
    rmode = machines_config(hostname)['mode']
    if 'parallelism' in cfg:
        mpi_per_slot = cfg['parallelism']['mpi_per_slot']

    OpenMP = True

    cores_per_node = machines_config(hostname)['queues'][queue]['cores_per_node']

    total_res = nodes * cores_per_node

    if OpenMP:
       ompthreads = math.floor(total_res/mpi_per_slot)
    else:
       ompthreads = 1

    print('available resources ',total_res)
    print('used resources ',mpi_per_slot * ompthreads)

    script = PrepareJob(exe=os.path.join(cfg['oops path'],'aq_error_covariance_training.x'),namelist=kwargs['jedinamelist'],
        mpiprocs=mpi_per_slot,ompthreads=ompthreads,
        mode=rmode,nodes=nodes,
        queue=queue,time=timedelta(hours=1),
        jobout=kwargs['jobout'],
        errout=kwargs['joberr']
        )
    with open(kwargs['jobname'],'w') as f:
        print(script,file=f)
    os.chmod(kwargs['jobname'], 0o755)
    

