#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Feb 12 14:55:37 2024

@author: eemili
"""

import os
import numpy as np
import matplotlib.pyplot as plt
from datetime import datetime as dt
from datetime import timedelta as td

inidate = '2024011300'
step = td(hours=24)
nsteps = 2

explist, exptype, expcolor = [], [], []
explist.append('/esarchive/exp/monarch/a5zv')
exptype.append('fc')
expcolor.append('navy')
explist.append('/esarchive/exp/monarch/a6vz')
exptype.append('da')
expcolor.append('crimson')
explist.append('/esarchive/exp/monarch/a6tl')
exptype.append('fc')
expcolor.append('teal')

outdir = '/esarchive/scratch/eemili/oper-da'
if not os.path.exists(outdir): os.makedirs(outdir)

inidate = dt.strptime(inidate, '%Y%m%d%H')
dates = [inidate+i*step for i in range(nsteps)]

fig = plt.figure(figsize=(12,20),dpi=250)
fig.autofmt_xdate()
axnobs = fig.add_subplot(311)

axnobs.set_title("Number of %s observations" % ('VIIRS'))
    
axnobs.grid(True)   
# axnobs.axis(xmin=dates[0],xmax=dates[-1])
axnobs.set_xlabel('Time (UTC)')
axnobs.legend(loc=1)
    
axobs = fig.add_subplot(312)
axobs.set_title("%s statistics (%s)" % ('VIIRS DOD','unitless'))
axobs.set_ylabel("%s" % ('unitless'))
                     
axobs.grid(True)   
# axobs.axis(xmin=dates[0],xmax=dates[-1])
axobs.set_xlabel('Time (UTC)')
axobs.legend(loc=1)

axdep = fig.add_subplot(313)
axdep.set_title("%s statistics (%s)" % ('VIIRS DOD','unitless'))
axdep.set_ylabel("%s" % ('unitless'))
    
axdep.grid(True)
# axdep.axis(xmin=dates[0],xmax=dates[-1])
axdep.set_xlabel('Time (UTC)')
axdep.legend(loc=1)
    
for idate,date in enumerate(dates):
    
    for iexp,exp in enumerate(explist):
        
        npfile = os.path.join(explist[iexp],'plots',date.strftime('%Y%m%d'),
                              'VIIRS_%s_time_series.npz' % (date.strftime('%Y%m%d')))
        
        if os.path.exists(npfile):
            
            d = np.load(npfile,allow_pickle=True)
            
            if exptype[iexp] == 'da':
                
                axnobs.plot(d['time'],d['nusedobs'],color='navy',
                        ls='-',lw=2,marker=None,label='Assimilated',markersize=4.0)
                axnobs.plot(d['time'],d['nrejobs'],color='black',
                    ls='--',lw=2,marker=None,label='Rejected',markersize=3.0)
                axnobs.plot(d['time'],d['ntotobs'],color='black',
                        ls='-',lw=2,marker=None,label='Total',markersize=4.0)
                
                axobs.errorbar(d['time'],np.ma.masked_values(d['y_avg'],d['missing_value']),
                            yerr=np.ma.masked_values(d['y_std'],d['missing_value']).filled(0.0),color='black',
                        ls=None,marker='o',label='y_da',markersize=4.0,
                        capsize=6, elinewidth=1.7, markeredgewidth=1.7)
                axobs.errorbar(d['time'],np.ma.masked_values(d['hofxfg_avg'],d['missing_value']),
                            yerr=np.ma.masked_values(d['hofxfg_std'],d['missing_value']).filled(0.0),color='salmon',
                        ls='--',marker='o',label='hofxfg',markersize=4.0,
                        capsize=6, elinewidth=1.7, markeredgewidth=1.7)
                axobs.errorbar(d['time'],np.ma.masked_values(d['hofxan_avg'],d['missing_value']),
                            yerr=np.ma.masked_values(d['hofxan_std'],d['missing_value']).filled(0.0),color=expcolor[iexp],
                        ls=None,marker='o',label='hofxan',markersize=4.0,
                        capsize=4, elinewidth=1.3, markeredgewidth=1.3)
                
                axdep.errorbar(d['time'],np.ma.masked_values(d['fgdep_avg'],d['missing_value']),
                            yerr=np.ma.masked_values(d['fgdep_std'],d['missing_value']).filled(0.0),color='salmon',
                        ls='--',marker='o',label='fgdep',markersize=4.0,
                        capsize=6, elinewidth=1.7, markeredgewidth=1.7)
                axdep.errorbar(d['time'],np.ma.masked_values(d['andep_avg'],d['missing_value']),
                            yerr=np.ma.masked_values(d['andep_std'],d['missing_value']).filled(0.0),color=expcolor[iexp],
                        ls=None,marker='o',label='andep',markersize=4.0,
                        capsize=4, elinewidth=1.3, markeredgewidth=1.3)
                
            if exptype[iexp] == 'fc':
                
                axnobs.plot(d['time'],d['nobs'],color=expcolor[iexp],
                        ls='--',lw=2,marker=None,label='Forecast',markersize=4.0)
                
                axobs.errorbar(d['time'],np.ma.masked_values(d['y_avg'],d['missing_value']),
                            yerr=np.ma.masked_values(d['y_std'],d['missing_value']).filled(0.0),color='black',
                        ls='--',marker='o',label='y_fc',markersize=4.0,
                        capsize=4, elinewidth=1.3, markeredgewidth=1.3)
                axobs.errorbar(d['time'],np.ma.masked_values(d['hofxfc_avg'],d['missing_value']),
                            yerr=np.ma.masked_values(d['hofxfc_std'],d['missing_value']).filled(0.0),color=expcolor[iexp],
                        ls=None,marker='o',label='hofxfc',markersize=4.0,
                        capsize=6, elinewidth=1.7, markeredgewidth=1.7)
                
                axdep.errorbar(d['time'],np.ma.masked_values(d['fcdep_avg'],d['missing_value']),
                            yerr=np.ma.masked_values(d['fcdep_std'],d['missing_value']).filled(0.0),color=expcolor[iexp],
                        ls=None,marker='o',label='fcdep',markersize=4.0,
                        capsize=6, elinewidth=1.7, markeredgewidth=1.7)
                       
plt.savefig(os.path.join(outdir,'VIIRS_time_series.png'))
plt.close(fig)