# coding: utf8
""" @author: plum """

import os
import fnmatch
import time
import datetime
import h5py
from pyhdf.SD import *
import numpy.ma as ma
import numpy as np

from AOD_instr import AODInstrument
from obsdb import daterange
import netCDF4
import ipdb

class AERUSGEO_AOD(AODInstrument):
    """ Class for AERUSGEO AOD data """

    def __init__(self, name, **kwargs):
        AODInstrument.__init__(self, name, **kwargs)

    def Convert(self, period, inputdir, meta_inputdir, dset_name, wavelength,
                **kwargs):
        """ Reads necessary data from files

        """
        
        nbdays = (period[1] - period[0]).days
        dates = (period[0] + datetime.timedelta(d) for d in range(nbdays))
        file_list = []
        
        # get Coordinates from metafile
        ds_geo = netCDF4.Dataset(meta_inputdir)
        lat = ds_geo.variables["Latitude"][:]
        lon = ds_geo.variables["Longitude"][:]
        
        scale_factor = 1
        
        for d in dates:
            f = inputdir + "SEV_AERUS-AEROSOL-D3_{date}_V1.04.nc".format(
                    date=d.strftime('%Y-%m-%d'))
            
            ds = netCDF4.Dataset(f)
            
            # get Scaling Factor
            if 'Scaling_Factor' in ds.variables[dset_name].ncattrs():
                scale_factor = ds.variables[dset_name].getncattr(
                        'Scaling_Factor')
            else :
                scale_factor = 1.

            # get data 
            data = ds.variables[dset_name][:]/scale_factor
            age = ds.variables["Z_Age"][:]
            #qflag = ds.variables["Q_Flag"][:]            
            #angle = ds.variables["ANGS_06_16"][:]
            #ipdb.set_trace() 
            
            # Set empty mask
            mask = np.ma.make_mask_none(data.shape)

            # mask negative values and values above filter_max
            if 'filter_max' in kwargs :
                filter_max = int(kwargs['filter_max'])
                mask_value = (data <= 0)|(data > filter_max)
            else :
                mask_value = (data <= 0)
            mask = np.ma.mask_or(mask, mask_value)
            
            # mask data above age threshold
            if 'age_threshold' in kwargs :
                age_threshold = int(kwargs['age_threshold'])
                mask_age = (age < 0)|(age > age_threshold)
                mask = np.ma.mask_or(mask, mask_age)
            
           # Apply mask on dataset 
            # Select only unmasked data
            data.mask = mask
            lat.mask = mask
            lon.mask = mask 
            vect_data = data.compressed()
            vect_lat = lat.compressed()
            vect_lon = lon.compressed()
            
            # get number of observations
            nobs = vect_data.size
            # define time 
            t = d+datetime.timedelta(hours=12)
            listdate = [t for i in range(nobs)]
            
            # wavelength
            wv = str(wavelength[0])
            
            #ipdb.set_trace()                        
            # Output variables
            self.nobs = self.nobs + nobs
            self.aod[wv] = np.ma.concatenate(
                    (self.aod[wv], 
                     np.ma.array(vect_data, dtype=self.aod[wv].dtype)))
            self.aod[wv].fill_value = self.missing_value
            self.lats = np.ma.concatenate(
                    (self.lats, np.ma.array(vect_lat, dtype=self.lats.dtype)))
            self.lats.fill_value = self.missing_value
            self.lons = np.ma.concatenate(
                    (self.lons, np.ma.array(vect_lon, dtype=self.lons.dtype)))
            self.lons.fill_value = self.missing_value
            self.mask = np.concatenate(
                    (self.mask, np.zeros((nobs), dtype=bool)))
            self.time = np.ma.concatenate(
                    (self.time, np.ma.array(listdate, dtype=self.np_typetime)))
            

def find_files(directory, pattern):
    for root, dirs, files in os.walk(directory):
        for basename in files:
            if fnmatch.fnmatch(basename, pattern):
                filename = os.path.join(root, basename)
                yield filename
