#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Tue Jan 16 17:02:31 2018

@author: emili
"""

# -*- coding: utf-8 -*-
"""
Surface observation class

Created on Thu Feb  2 17:37:58 2017

@author: emili
"""

import os
import numpy as np
import h5py
import glob
from datetime import datetime,timedelta
import seaborn as sns
import matplotlib.pyplot as plt
from colconc_instr import ColConcInstrument

class GOME2_O3M(ColConcInstrument):
    # Class for KNMI GOME2 O3 profiles
    def __init__(self,name,**kwargs):

        kwargs["species"] = ["O3"]
        if "nmaxlevs" not in kwargs: kwargs["nmaxlevs"] = 40
        self.version = "4.50" # Corresponds to the format version
        
        ColConcInstrument.__init__(self,name,**kwargs)
        
        self.press_isconst = False
        self.press_coorddir = 'downward'
        
        self.hasavk = True
        self.hascov = True
        
        self.auxvars_1D = ['CloudFraction']
        
    def Convert(self,period,inputdir,**kwargs):
        
        # Default options
        timebnd, latbnd, lonbnd = (period[0],period[1]), None, None
        
        if 'TimeBnd' in kwargs.keys(): timebnd = kwargs['TimeBnd']
        if 'LatBnd' in kwargs.keys(): latbnd = kwargs['LatBnd']
        if 'LonBnd' in kwargs.keys(): lonbnd = kwargs['LonBnd']

        firstday = datetime(period[0].year,period[0].month,period[0].day)
        dates = [firstday + timedelta(days=i) for i in range((period[1]-period[0]).days+1)]
        for date in dates: 
            for f in sorted(glob.glob(os.path.join(inputdir,"%4i" % (date.year),"%02i" % (date.month),"%02i" % (date.day),"S-O3M_GOME*.hdf5"))):
                self.Convert_OPERA_L2(f,timebnd,latbnd,lonbnd)
        
    def Select(self,**kwargs):
        
        ClearSky = False
        
        if 'ClearSky' in kwargs: ClearSky = True

        if self.nobs > 0:
            
            if ClearSky is True:
                clftrshold = 0.20
                if "CloudThreshold" in kwargs["ClearSky"]: clftrshold = kwargs["ClearSky"]["CloudThreshold"]
                clearskymsk = np.ma.masked_greater(self.CloudFraction,clftrshold).mask
                if clearskymsk.any(): 
                    print ">>> Masking %i pixels over %i with cloud fraction > %i percent" % (clearskymsk.sum(),self.nobs,clftrshold*100)
                    self.mask = np.ma.mask_or(self.mask,clearskymsk)
        
            ColConcInstrument.Select(self,**kwargs)
            
            # Update the masks for fields that are not part of ColConcInstrument
            if self.mask.any():
                print ">>> Retained %i pixels at the end of the selection" % ((~self.mask).sum())
                for var in self.auxvars_1D: 
                    if var in self.__dict__.keys(): self.__dict__[var].mask = self.mask

    def PlotO3MAux(self,outdir="./",**kwargs):
        
        lkwargs = {}
        figsize = (12,12)
        markersize = 2
        colormap = "YlOrBr"
            
        if 'ObsOnMapOpt' in kwargs: lkwargs = kwargs["ObsOnMapOpt"]
        if "figsize" in lkwargs: figsize = lkwargs["figsize"]
        if 'markersize' in lkwargs: markersize = lkwargs['markersize']
            #if 'colormap' in kwargs["ObsOnMapOpt"]: colormap = kwargs["ObsOnMapOpt"]['colormap']
            
        #varlist = self.auxvars_1D + ["Determinant","ConditionNumber"]
        varlist = self.auxvars_1D
        
        fig = plt.figure(figsize=figsize,dpi=200)
        
        filename = os.path.join(outdir,"%s_AuxMap_%s_%s" % \
                            (self.name,self.time.compressed().min().astype(str)[:13].replace("-",""), \
                             self.time.compressed().max().astype(str)[:13].replace("-","")))
        
        nsubplt = int(np.ceil(np.sqrt(len(varlist))))
        
        for ivar,var in enumerate(varlist):
            
            ax = fig.add_subplot(nsubplt,nsubplt,ivar+1)

            if var == "Determinant":
                fld = np.ma.array(np.linalg.det(self.cov["O3"]))
            elif var == "ConditionNumber":
                fld = np.ma.array(np.linalg.cond(self.cov["O3"]))
            else:
                fld = self.__dict__[var]
            
            if fld.count() == 0: return
            
            # Plot only values corresponded to retained obs
            lmask = self.mask | fld.mask
            
            # Plot all values 
            #fld.fill_value = 999
            #fld = fld.filled() 
            #lmask = np.zeros((fld.size),dtype=bool)
            
            vmin = fld[~lmask].min()
            vmax = fld[~lmask].max()
            
            ax.set_title("%s \n Avg: %.2f,Min: %.2f,Max: %.2f" % (var,fld[~lmask].mean(),vmin,vmax))
            
            # Variable related keywords have priority over global keywords
            lvarkwargs = {}

            if var in lkwargs: 
                lvarkwargs = lkwargs[var]
            else:
                lvarkwargs = lkwargs
            
            if "range" in lvarkwargs:
                vmin = lvarkwargs["range"]["min"]
                vmax = lvarkwargs["range"]["max"]
            
            m = self.SetBaseMap(ax,**kwargs)
            
            sc = m.scatter(self.lons[~lmask],self.lats[~lmask],markersize,marker='o', \
                       c=fld[~lmask],\
                       cmap=colormap,vmin=vmin,vmax=vmax,\
                       latlon=True,zorder=10)

            plt.colorbar(sc,orientation='horizontal')
                
        fig.savefig(filename)
        plt.close(fig)
        
    # A comprehensive plotter for GOME2-O3M data
    def Plot(self,outdir="./",**kwargs):
    
        sns.set() # Apply seaborn beautificator
        ColConcInstrument.Plot(self,outdir=outdir,**kwargs)
        # Additional plots (only available when converting data)
        if self.nobs > 0:
            if 'makeplots' in kwargs:
                if 'O3MAux' in kwargs['makeplots']: self.PlotO3MAux(outdir=outdir,**kwargs)
        else:
            print '>> No obs to be plotted'
            
    def Convert_OPERA_L2(self,f,timebnd,latbnd=None,lonbnd=None):

        vars2D = {'col':'StateRetrieved','prior':'Apriori'}
        vars3D = {'avk':'AveragingKernel'}
        
        start, end = os.path.basename(f).split("_")[5], os.path.basename(f).split("_")[6]
        dtstart = datetime.strptime(start[:-1],"%Y%m%d%H%M%S")
        dtend =  datetime.strptime(end[:-1],"%Y%m%d%H%M%S")
        # Just skip granules that are not in the timewindow, to avoid unuseful and costly codaread call
        if ( dtstart > timebnd[1] or dtend < timebnd[0] ): return
        print "> GOME2 KNMI granule reader: working on %s " % (f)
                        # Read time, lat, lon and obtain the right index to extract further data
        fid = h5py.File(f,"r")
        
        geolocgrp = "/GEOLOCATION/"
        datagrp = "/DATA/"
        
        lats = np.ma.array(fid[geolocgrp+"LatitudeCenter"][:])
        lons = np.ma.array(fid[geolocgrp+"LongitudeCenter"][:])
        time = np.array([datetime.strptime(t[:19],"%Y-%m-%dT%H:%M:%S") for t in fid[geolocgrp+"Time"][:]],dtype=self.np_typetime)
        qltyinput = fid[datagrp+"QualityInput"][:]
        qltyproc  = fid[datagrp+"QualityProcessing"][0,:].flatten() # Use global quality flag
        nstate = fid[datagrp+"NState"][:]
        # Compute indexes for further data extraction without reading unwanted data
        # First do a time and geographical screening
        # isinside = True if data is retained (inverse of np.ma masks)
        isinside = np.full((time.size,), True, dtype=bool)

        timemsk = ((time > timebnd[0]) & (time < timebnd[1]))
        isinside = np.logical_and(isinside,timemsk)
        
        if latbnd is not None: 
            latmsk = np.ma.masked_outside(lats,latbnd[0],latbnd[1],copy=False).mask
            isinside = np.logical_and(isinside,~latmsk)
        if lonbnd is not None: 
            lonmsk = np.ma.masked_outside(lons,lonbnd[0],lonbnd[1],copy=False).mask
            isinside = np.logical_and(isinside,~lonmsk)
        
        retmask = (qltyproc == 1) # Keep and further process only pixels with valid retrievals
        isinside = np.logical_and(isinside,retmask)
        
        statemsk = (nstate == 42) # Keep and further process only pixels with valid retrievals
        isinside = np.logical_and(isinside,statemsk)
        
        spc = self.species[0]
        
        # Read further data and store them in python structures
        if isinside.any():
            
            self.nobs += isinside.sum()
            self.time = np.ma.concatenate((self.time,time[isinside.nonzero()]))
            self.lats = np.ma.concatenate((self.lats,np.ma.array(lats[isinside.nonzero()],dtype=self.lats.dtype)))
            self.lons = np.ma.concatenate((self.lons,np.ma.array(lons[isinside.nonzero()],dtype=self.lons.dtype)))
            
            # Extract vars
            lvar = fid[datagrp+'OutputPressureGrid'][:,isinside.nonzero()[0]]
            lvar = np.ma.array(lvar[::-1,:],dtype=self.press.dtype,fill_value=self.missing_value)
            self.press = np.ma.array(np.ma.concatenate((self.press,lvar.transpose()*100)),fill_value=self.missing_value)
            
            # Extract and fill remaining vars
            for key, var in vars2D.iteritems():
                lvar = fid[datagrp+var][:self.nmaxlevs,isinside.nonzero()[0]]
                lvar = np.ma.array(lvar[::-1,:],dtype=self.__dict__[key][spc].dtype,fill_value=self.missing_value)
                self.__dict__[key][spc] = np.ma.array(np.ma.concatenate((self.__dict__[key][spc],lvar.transpose())),fill_value=self.missing_value)
                
            #covout = np.transpose(fid[datagrp+'ErrorCovarianceNoise'][:self.nmaxlevs,:self.nmaxlevs,isinside.nonzero()[0]],axes=(2,0,1))
            # Very difficult to find reasonable inverse for the error covariance
            nmaxcov = self.nmaxlevs # <= 40
            covin = np.transpose(fid[datagrp+'ErrorCovarianceNoise'][:nmaxcov,:nmaxcov,isinside.nonzero()[0]],axes=(2,0,1))[:,::-1,::-1]
            covout = np.repeat(np.eye(self.nmaxlevs)[None,:,:],isinside.sum(),axis=0)
            #covout[:,:nmaxcov,:nmaxcov] = covin # Use this if you want to keep original R in DAIMON hdf5
            
            # Transpose and inverse R before filling the numpy masked array
            inverted = np.ones(isinside.sum(),dtype=bool)
            
            # Some diagnostics of input matrices
            symmetric = np.array([np.allclose(covin[i,:,:],covin[i,:,:].T,rtol=1e-5) for i in range(isinside.sum())],dtype=bool)
            cond = np.linalg.cond(covin)
            det = np.linalg.det(covin)
            print "R diagnostics:"
            print "%i covariances over %i pixels are symmetric" % (symmetric.sum(),isinside.sum())
            print "Determinant (min,max,mean):" , det.min(), det.max(), det.mean()
            print "Condition number (min,max,mean):" , cond.min(), cond.max(), cond.mean()
            
            try:
                covout[:,:nmaxcov,:nmaxcov] = np.linalg.pinv(covin,rcond=1e-5)
            except np.linalg.LinAlgError:
                print ">> Error with matrix inversion. Skip all profiles from this granule."
                inverted[:] = False
                
#            covout = covin
            print "R Inversion diagnostics:"
            print "Correctly inverted %i covariances over %i pixels" % (inverted.sum(),isinside.sum())

            self.cov[spc] = np.ma.array(np.ma.concatenate((self.cov[spc],np.ma.array(covout,dtype=self.cov[spc].dtype))),fill_value=self.missing_value)
            
            for key, var in vars3D.iteritems():
                lvar = fid[datagrp+var][:self.nmaxlevs,:self.nmaxlevs,isinside.nonzero()[0]]
                lvar = np.ma.array(lvar[::-1,::-1,:],dtype=self.__dict__[key][spc].dtype,fill_value=self.missing_value)
                self.__dict__[key][spc] = np.ma.array(np.ma.concatenate((self.__dict__[key][spc],np.ma.transpose(lvar,axes=(2,1,0)))),fill_value=self.missing_value)

            for var in self.auxvars_1D:
                locals()[var] = np.ma.array(fid[datagrp+var][:])[isinside.nonzero()]
                if var not in self.__dict__.keys(): self.__dict__[var] = np.ma.empty((0,),dtype=locals()[var].dtype) # Init the variable
                self.__dict__[var] = np.ma.concatenate((self.__dict__[var],locals()[var]))
                                        
            mask = np.zeros((isinside.sum()),dtype=bool) # By default all obs are accepted
                                    
            # Mask pixels with missing cloud fraction (assumes cloudfraction was extracted above)
            mask[locals()['CloudFraction'] < 0] = True
            
            # Mask pixels with bad matrix inversion
            mask[inverted == False] = True
                            
            self.mask = np.concatenate((self.mask,mask))
            
        fid.close()