# coding: utf8
""" @author: plum """

import os
import fnmatch
import time
from datetime import datetime, time, timedelta

import h5py
#from pyhdf.SD import *
import numpy.ma as ma
import numpy as np

from AOD_instr import AODInstrument
from obsdb import daterange

from netCDF4 import Dataset, MFDataset, num2date, date2num


class PMAP_AOD(AODInstrument):
    """ Class for PMAP AOD data
    """

    def __init__(self, name, **kwargs):
        AODInstrument.__init__(self, name, **kwargs)

    def Convert(self, period, inputdirs, **kwargs):
        """ Reads necessary data from files

        Works for a single wavelength
        Observations are kept if quality flag is below QAFlag - see PDF_PMAP_PFS.pdf

        """
        if isinstance(inputdirs, basestring):
            inputdirs = [inputdirs]  # to make sure that inputdir is iterable as a sequence in a for loop (that it's a list, and not a string)
        if "Qflag" in kwargs:
            Qflag = kwargs["Qflag"]
        else:
            Qflag = 0

        wv = self.wllist[0]  # Single wavelength

        for dir in inputdirs:
            for d in daterange(period[0], period[1]):
                # Search for file name
                yyyy = str(d.timetuple().tm_year)
                mm = str(d.timetuple().tm_mon).rjust(2,'0')
                dd = str(d.timetuple().tm_mday).rjust(2,'0')
                print('M0*' + yyyy + mm + dd + '*.nc')
                for filename in find_files(dir,'M0*' + yyyy + mm + dd + '*.nc'):
                    indexdate = filename.find("-"+yyyy+mm+dd)
                    obsdate = filename[indexdate+1:indexdate+13]

                    # open and read filename
                    f = Dataset(filename)
                    val = f.groups['Data']['MeasurementData']['ObservationData']['Aerosol']['aerosol_optical_depth'][:].tolist()
                    flag = f.groups['Data']['MeasurementData']['ObservationData']['Aerosol']['aerosol_retrievals_flag'][:].tolist()
                    lat = f.groups['Data']['MeasurementData']['GeoData']['aerosol_center_latitude'][:].tolist()
                    lon = f.groups['Data']['MeasurementData']['GeoData']['aerosol_center_longitude'][:].tolist()
                    time = f.groups['Data']['MeasurementData']['GeoData']['aerosol_sensor_readout_start_time'][:].tolist()

                    #Filtering data
                    thr=0.0
                    latm=[lat[i] for i in range(len(val)) if val[i]>thr and flag[i]<=Qflag]
                    lonm=[lon[i] for i in range(len(val)) if val[i]>thr and flag[i]<=Qflag]
                    timem=[int(time[i]) for i in range(len(val)) if val[i]>thr and flag[i]<=Qflag]
                    AODm=[val[i] for i in range(len(val)) if val[i]>thr and flag[i]<=Qflag]

                    Nobs = len(latm)

                    # Time values
                    #We add 946684800, which is the timestamp of 2000-01-01 00 UTC
                    listdate = [datetime.fromtimestamp(timem[i]+946684800) for i in range(Nobs)]

                    print("Instant ", obsdate, " ", Nobs, "values kept after applying QAflag =", Qflag)

                    # Output variables
                    self.nobs = self.nobs + Nobs
                    self.aod[wv] = np.ma.concatenate((self.aod[wv], np.ma.array(AODm, dtype=np.float32)))
                    self.lats = np.ma.concatenate((self.lats, np.ma.array(latm, dtype=self.lats.dtype)))
                    self.lons = np.ma.concatenate((self.lons, np.ma.array(lonm, dtype=self.lons.dtype)))
                    self.mask = np.concatenate((self.mask, np.zeros(Nobs, dtype=bool)))  # keep all
                    self.aod[wv].mask = self.mask
                    self.time = np.ma.concatenate((self.time, np.ma.array(listdate, dtype=self.np_typetime)))


def find_files(directory, pattern):
    for root, dirs, files in os.walk(directory):
        for basename in files:
            if fnmatch.fnmatch(basename, pattern):
                filename = os.path.join(root, basename)
                yield filename
