# coding: utf8

from __future__ import print_function

__doc__ = """
"""

__author__ = "Sbastien Massart - Mars 2010"

#------------------------------
# IMPORT DES MODULES 
#------------------------------
#
import os
# import string
import numpy
#
#-------------------------------------------------
# SUBROUTINE DE LECTURE DES FICHIER HOBS
#-------------------------------------------------
#
def LecHOBS (filename, obstype='profile')  :
  """ Lecture des fichiers HOBS au format MCG"""
#
# 1. Ouverture, lecture et fermeture du fichier  
# ------------------------------------------------
#
  if not os.path.exists(filename):
    print(filename + " not found")
    return [], [], [], [], [], [], []

  file = open(filename,'r')
  lines=file.readlines()
  file.close()
#
# 2. Lecture du nombre de profils
#-------------------------------------------------
#
  nobs = int(lines[0].split()[0])
  ostype = lines[1]
  print(">> Found ", nobs, " obs for ", ostype)
#
# 3. Lecture des profils
#-------------------------------------------------
#
#  3.1. Initialisation des tableaux
#
  date = []
  name = []
  lat = []
  lon = []
  vmr = []
  press = []
#
  iline = 2
  if obstype is 'partcol': 
    str_list = [stri for stri in ostype.split() if stri[:6].upper() == 'INTQTY']
    nbniv = int(str_list[0].split('=')[-1])
#
#  3.2. Boucle sur les observations
#  
  for iobs in range(nobs) : 
#
#     Lecture de la 1er ligne de chaque profile
#  
    line = lines[iline].split()
#
#     Lecture de la date, latitude, longitude et nombre de niveau de pression
#  
    date.append(line[0])
    name.append(line[1:3])
    lat.append(float(line[3]))
    llon = float(line[4])
    if llon > 180. : 
      lon.append(llon - 360.)
    else : 
      lon.append(llon)    
#
#     Lecture des pressions et des concentrations
#  
    vmr.append([])
    press.append([])
    iline = iline + 1 
    if obstype is 'profile':
      nbniv = int(line[5]) 
      for iniv in range(nbniv) :
        press[iobs].append(float(lines[iline].split()[0]))
        vmr[iobs].append(float(lines[iline+1].split()[-1]))
        iline = iline + 2 
    elif obstype is 'partcol':  
      press[iobs].append(float(lines[iline].split()[0]))
      vmr[iobs].append(float(lines[iline+1].split()[-1]))
      iline = iline + 2 
      for iniv in range(1,nbniv) :
        press[iobs].append(float(lines[iline].split()[0]))
        iline = iline + 2 
    elif obstype is 'totcol':    
      line = lines[iline].split()
      press[iobs].append([float(line[0]), float(line[1])])
      line = lines[iline+1].split()
      vmr[iobs].append(float(line[-1]))
      iline = iline + 2 
    else:
      print("Observation type ", obstype, " not implemented")
    
#
# 4. Retour des variables calculees
# ------------------------------------------------
#
  return ostype, date, name, lat, lon, vmr, press
#
#-------------------------------------------------
# SUBROUTINE DE LECTURE DES FICHIER HOBS
#-------------------------------------------------
#
def LecHAVK (filename)  :
  """ Lecture des fichiers HAVK au format MCG"""
#
# 1. Ouverture, lecture et fermeture du fichier  
# ------------------------------------------------
#
  if not os.path.exists(filename):
    print(filename + " not found")
    return [], [], [], [], [], []

  file = open(filename,'r')
  lines=file.readlines()
  file.close()
#
# 2. Lecture du nombre de profils
#-------------------------------------------------
#
  nobs = int(lines[0].split()[0])
  ostype = lines[1]
#
# 3. Lecture des profils
#-------------------------------------------------
#
#  3.1. Initialisation des tableaux
#
  date = []
  name = []
  lat = []
  lon = []
  vmr = []
#
  iline = 2
#
#  3.2. Boucle sur les observations
#  
  for iobs in range(nobs) : 
#
#     Lecture de la 1er ligne de chaque profile
#  
    line = lines[iline].split()
#
#     Lecture de la date, latitude, longitude et nombre de niveau de pression
#  
    date.append(line[0])
    name.append(line[1:3])
    lat.append(float(line[3]))
    llon = float(line[4])
    if llon > 180. : 
      lon.append(llon - 360.)
    else : 
      lon.append(llon)
#
#     Lecture des nombre de niveaux
#  
    iline += 1
    line = lines[iline].split()
    nbniv = int(line[1])
    vmr.append([])
    iline = iline + 1 
    for iniv in range(nbniv) :
      vmr[iobs].append(float(lines[iline].split()[-1]))
      iline = iline + 1 
#
# 4. Retour des variables calculees
# ------------------------------------------------
#
  return ostype, date, name, lat, lon, vmr
#
#-------------------------------------------------
# SUBROUTINE DE LECTURE DES FICHIER HCOV
#-------------------------------------------------
#
def LecHCOV (filename, obstype='profile')  :
  """ Lecture des fichiers HCOV """
#
# 1. Ouverture, lecture et fermeture du fichier  
# ------------------------------------------------
#
  file = open(filename,'r')
  lines=file.readlines()
  file.close()
#
# 2. Lecture du nombre de profils
#-------------------------------------------------
#
  nobs = int(lines[0].split()[0])
  ostype = lines[1]
#
# 3. Lecture des profils
#-------------------------------------------------
#
  date = []
  name = []
  lat = []
  lon = []
  vmr = []
  press = []
#
  iline = 2
  for iobs in range(nobs) : 
#
#     Lecture de la 1er ligne de chaque profile
#  
    line = lines[iline].split()
#
#     Lecture de la date, latitude, longitude et nombre de niveau de pression
#  
    date.append(line[0])
    name.append(line[1:3])
    lat.append(float(line[3]))
    llon = float(line[4])
    if llon > 180. : 
      lon.append(llon - 360.)
    else : 
      lon.append(llon)
    nbniv = int(line[5]) 
#
#     Lecture des pressions et des concentrations d'ozone
#  
    vmr.append(numpy.zeros((nbniv,nbniv),float))
    press.append([])
    iline = iline + 1 
    for iniv in range(nbniv) :
      press[iobs].append(float(lines[iline].split()[0]))
      for jniv in range(iniv+1) :
        iline = iline + 1 
        cov = float(lines[iline].split()[-1])
        vmr[iobs][iniv,jniv] = cov
        vmr[iobs][jniv,iniv] = cov
      iline = iline + 1 
#
#  4. Retour des variables calculees
# ------------------------------------------------
#
  return ostype, date, name, lat, lon, vmr, press
#
#-------------------------------------------------
# SUBROUTINE DE LECTURE DES FICHIER HDOM
#-------------------------------------------------
#
def LecHDOM (filename, totcol=False, superobs=False)  :
  """ Lecture des fichiers HDOM au format MCG"""
#
# 1. Ouverture, lecture et fermeture du fichier  
# ------------------------------------------------
#
  print('>> Lecture du fichier ', os.path.basename(filename))
  file = open(filename,'r')
  lines=file.readlines()
  file.close()
#
# 2. Lecture du nombre de profils
#-------------------------------------------------
#
#  nobs = int(string.split(lines[0])[0]) -1
  print(lines[0])
  nobs = int(lines[0].split()[0])
  ninst = int(lines[0].split()[1])
  if nobs != 0 :
     ostype = lines[1]
  else :  
     ostype = None

  print('Found ', nobs, ' observations for ', ninst, ' instruments')
#
# 3. Lecture des profils
#-------------------------------------------------
#
#  3.1. Initialisation des tableaux
#
  date = []
  name = []
  lat = []
  lon = []
  vmr = []
  obs = []
  use = []
  press = []
#
  iline = 1+ninst
  
#
#  3.2. Boucle sur les observations
#  
  try :
    for iobs in range(nobs) : 
    #print(iobs)
    #
    #     Lecture de la 1er ligne de chaque profile
    #  
      line = lines[iline].split()
      #
      #       Lecture de la date, latitude, longitude et nombre de niveau de pression
      #    
      date.append(line[0])
      name.append(line[1:3])
      lat.append(float(line[3]))
      llon = float(line[4])
      if llon > 180. : 
        lon.append(llon - 360.)
      else : 
        lon.append(llon)
      nbniv = int(line[5]) 
      #
      #       Lecture des pressions et des concentrations d'ozone
      #    
      vmr.append([])
      obs.append([])
      use.append([])
      press.append([])
      iline = iline + 1 
      if not totcol :
        for iniv in range(nbniv) :
          press[iobs].append(float(lines[iline].split()[0]))
          vmr[iobs].append(float(lines[iline+1].split()[2]))
          obs[iobs].append(float(lines[iline+1].split()[3]))
          use[iobs].append(lines[iline+1].split()[6])
          iline = iline + 2 
      else :    
        line = lines[iline].split()
        press[iobs].append([float(line[0]), float(line[1])])
        line = lines[iline+1].split()
        vmr[iobs].append(float(line[-5]))
        obs[iobs].append(float(line[-4]))
        use[iobs].append(line[-1])
        iline = iline + 2 
#      
  except IndexError:
    print('    >  Probleme avec le fichier')
#
# Modification pour GOMOS 
#    press[iobs].reverse()      
#    vmr[iobs].reverse()     
#
# 4. Retour des variables calculees
# ------------------------------------------------
#
  return ostype, date, name, lat, lon, vmr, obs, use, press
#-------------------------------------------------
# SUBROUTINE DE LECTURE DES FICHIER HOBS
#-------------------------------------------------
#
def LecHDIAG (filename, totcol=False)  :
  """ Lecture des fichiers HDIAG au format MCG"""
#
# 1. Ouverture, lecture et fermeture du fichier  
# ------------------------------------------------
#
  file = open(filename,'r')
  lines=file.readlines()
  file.close()
#
# 2. Lecture du nombre de profils
#-------------------------------------------------
#
  nobs = int(lines[0].split()[0])
  ostype = lines[1]
#
# 3. Lecture des profils
#-------------------------------------------------
#
#  3.1. Initialisation des tableaux
#
  date = []
  name = []
  lat = []
  lon = []
  vmr = []
  obs = []
  dif = []
  press = []
#
  iline = 2
#
#  3.2. Boucle sur les observations
#  
  for iobs in range(nobs) : 
#
#     Lecture de la 1er ligne de chaque profile
#  
    line = lines[iline].split()
#
#     Lecture de la date, latitude, longitude et nombre de niveau de pression
#  
    date.append(line[0])
    name.append(line[1:3])
    lat.append(float(line[3]))
    llon = float(line[4])
    if llon > 180. : 
      lon.append(llon - 360.)
    else : 
      lon.append(llon)
    nbniv = int(line[5]) 
#
#     Lecture des pressions et des concentrations d'ozone
#  
    vmr.append([])
    obs.append([])
    dif.append([])
    press.append([])
    iline = iline + 1 
    if not totcol :
      for iniv in range(nbniv) :
        line = lines[iline].split()
        press[iobs].append(float(line[0]))
        line = lines[iline+1].split()
        vmr[iobs].append(float(line[-3]))
        obs[iobs].append(float(line[-2]))
        dif[iobs].append(float(line[-1]))
        iline = iline + 2 
    else :    
      line = lines[iline].split()
      press[iobs].append([float(line[0]), float(line[1])])
      line = lines[iline+1].split()
      vmr[iobs].append(float(line[-3]))
      obs[iobs].append(float(line[-2]))
      dif[iobs].append(float(line[-1]))
      iline = iline + 2 
#
# 4. Retour des variables calculees
# ------------------------------------------------
#
  return ostype, date, name, lat, lon, vmr, obs, dif, press

# This functions shrink the species and obstype name to fit in
# HDAT files
def SetSpeciesString(name):
  
  newname = name.strip('_')
  if len(newname) > 3: newname = newname[:3]  
  
  return newname

def SetObsNameString(name):
  
  if len(name) < 6: 
    lname = 6 - len(name) 
    newname = lname * ' ' + name
  else :
    newname = name[0:6]

  return newname

def SetObsCodeString(name):
  
  if len(name) < 5: 
    lname = 5 - len(name) 
    newname = lname * ' ' + name
  else :
    newname = name[0:5]

  return newname
#
#-------------------------------------------------
# SUBROUTINE D'ECRITURE DES FICHIERS HOBS
#-------------------------------------------------
#
def WriteHOBS(filename, ostype, date, name, lat, lon, \
             species_name, vmr, press, obstype='profile', append=False, extra_code = '00000') :
#
# 1. Ouverture, lecture et fermeture du fichier  
# ------------------------------------------------
#  
  nobs = len(date)
  line = "%4i  %i   \n" % (nobs, 1)
  
  #Append data or write a new file header
  if append :
    file = open(filename,'a')
  else :
    file = open(filename,'w')
    file.write(line)
    #Add new line if not present in the header
    if ostype[-1:] == '\n': 
      file.write(ostype)
    else:
      file.write(ostype+'\n')
#
# 3. Ecriture des profils
#-------------------------------------------------
#
  new_sname = SetSpeciesString(species_name)
  
  print("Starting writing ", nobs, " obs")
#
  for iobs in range(nobs) : 
#
    nbniv = len(press[iobs])
#
#     Lecture de la 1er ligne de chaque profile
#  
    if isinstance(name,basestring) is True: 
      new_name = SetObsNameString(name)
    else:
      new_name = SetObsNameString(name[iobs])

    if isinstance(extra_code,basestring) is True: 
      new_code = SetObsCodeString(extra_code)
    else:
      new_code = SetObsCodeString(extra_code[iobs])
  
    line = '%s %s %s' % (date[iobs], new_name, new_code)
    if obstype == 'partcol':
      line = line + ' %6.2f %7.2f %2.0i\n' % (lat[iobs], lon[iobs], 1)    
    else:
      line = line + ' %6.2f %7.2f %2.0i\n' % (lat[iobs], lon[iobs], nbniv)    
    file.write(line)
#
#     Lecture des pressions et des concentrations d'ozone
#  
    for iniv in range(nbniv) :
      if obstype == 'profile': 
        line  = "%7d %2i\n" % (press[iobs][iniv], 1)          
        if len(vmr[iobs]) == 1 :
          line += "[%3s] %12.5e\n" % (new_sname, vmr[iobs][0])
        else : 
          line += "[%3s] %12.5e\n" % (new_sname, vmr[iobs][iniv])
      elif obstype == 'partcol' :
        line  = "%7d %2i\n" % (press[iobs][iniv], 1)
        line += "[%3s] %12.5e\n" % (new_sname, vmr[iobs][0])
      else:
        line  = "%7.0d %7.0d %2.0i\n" % (10., 100000., 1)
        line += "[%3s] %12.5e\n" % (new_sname, vmr[iobs])
      file.write(line)
    #file.flush()
#
# 4. Retour des variables calculees
# ------------------------------------------------
#
  file.close()
#
# 4. Retour des variables calculees
# ------------------------------------------------
#
  return
#
#-------------------------------------------------
# SUBROUTINE D'ECRITURE DES FICHIERS HAVK
#-------------------------------------------------
#
def WriteHAVK(filename, ostype, date, name, lat, lon, \
             species_name, avk, press, append=False, extra_code = '00000') :
#
# 1. Ouverture, lecture et fermeture du fichier  
# ------------------------------------------------
#  
  nobs = len(date)
  line = "%4i  %i   \n" % (nobs, 1)
  #Append data or write a new file header
  if append :
    file = open(filename,'a')
  else :
    file = open(filename,'w')
    file.write(line)
    #Add new line if not present in the header
    if ostype[-1:] == '\n': 
      file.write(ostype)
    else:
      file.write(ostype+'\n')
#
# 3. Ecriture des profils
#-------------------------------------------------
#
  print("Starting writing ", nobs, " obs")
#
  for iobs in range(nobs) : 
#
    nbniv = len(press[iobs])
#
#     Lecture de la 1er ligne de chaque profile
#  
    line = "%7.0f %2i\n" % (0,nbniv)

    if isinstance(name,basestring) is True: 
      new_name = SetObsNameString(name)
    else:
      new_name = SetObsNameString(name[iobs])

    if isinstance(extra_code,basestring) is True: 
      new_code = SetObsCodeString(extra_code)
    else:
      new_code = SetObsCodeString(extra_code[iobs])
    
    #Write the number of avk levels
    line = '%s %s %s' % (date[iobs], new_name, new_code)
    line = line + ' %6.2f %7.2f %2.0i\n' % (lat[iobs], lon[iobs], 1)    
    file.write(line)
    line = "%7.0f %2i\n" % (0,nbniv)
    file.write(line)
#
#     Lecture des pressions et des concentrations d'ozone
#  
    for iniv in range(nbniv) :
      line = "[ 1,%2i] %12.5e\n" % (iniv+1, avk[iobs][iniv])
      file.write(line)
    #file.flush()
#
# 4. Retour des variables calculees
# ------------------------------------------------
#
  file.close()
#
# 4. Retour des variables calculees
# ------------------------------------------------
#
  return 
#
#-------------------------------------------------
# SUBROUTINE D'ECRITURE DES FICHIERS HOBS
#-------------------------------------------------
#
def WriteHCOV(filename, ostype, date, name, lat, lon, species_name, vmr, press, \
                totcol=False, append=False, diagonal = True, extra_code = '00000') :
#
# 1. Ouverture, lecture et fermeture du fichier  
# ------------------------------------------------
#  
  nobs = len(date)
  line = "%4i  %i   \n" % (nobs, 1)
  #Append data or write a new file header
  if append :
    file = open(filename,'a')
  else :
    file = open(filename,'w')
    file.write(line)
    #Add new line if not present in the header
    if ostype[-1:] == '\n': 
      file.write(ostype)
    else:
      file.write(ostype+'\n')
#
# 3. Ecriture des profils
#-------------------------------------------------
#  
  for iobs in range(nobs) : 
#
    nbniv = len(press[iobs])
#
#     Lecture de la 1er ligne de chaque profile
#  
    if isinstance(name,basestring) is True: 
      new_name = SetObsNameString(name)
    else:
      new_name = SetObsNameString(name[iobs])

    if isinstance(extra_code,basestring) is True: 
      new_code = SetObsCodeString(extra_code)
    else:
      new_code = SetObsCodeString(extra_code[iobs])
    
    line = '%s %s %s' % (date[iobs], new_name, new_code)
    line = line + ' %6.2f %7.2f %2.0i\n' % (lat[iobs], lon[iobs], nbniv)    
    file.write(line)
#
#     Lecture des pressions et des concentrations d'ozone
#  
    for iniv in range(nbniv) :
      if totcol :
        line  = "%7.0d %7.0d %2.0i\n" % (10., 100000., 1)
      else :
        line = "%7d %2i\n" % (press[iobs][iniv], iniv+1)
      file.write(line)
      for jniv in range(iniv+1) :
        if diagonal and (iniv == jniv) :
          line = "[%2i,%2i] %10.4e\n" % (iniv+1, jniv+1, vmr[iobs][iniv])
        elif diagonal :
          line = "[%2i,%2i] %10.4e\n" % (iniv+1, jniv+1, 0.)
        else :
          line = "[%2i,%2i] %10.4e\n" % (iniv+1, jniv+1, vmr[iobs][iniv,jniv])
        file.write(line)
    file.flush()
#      for jniv in range(iniv) :
#        line = "[%2i,%2i] %10.4e\n" % (iniv+1, jniv+1, 0.)
#        file.write(line)
#      print(vmr[iobs][iniv])
#      line = "[%2i,%2i] %10.4e\n" % (iniv+1, iniv+1, vmr[iobs][iniv])
#      file.write(line)
#
# 4. Retour des variables calculees
# ------------------------------------------------
#
  file.close()
#
# 4. Retour des variables calculees
# ------------------------------------------------
#
  return 
  
#-------------------------------------------------
# SUBROUTINE D'ECRITURE DES FICHIERS HDOM
#-------------------------------------------------
#
def WriteHDOM(filename, ostype, date, name, lat, lon, \
             species_name, obs, vmr, press, totcol=False, append=False, extra_code = '00000') :
#
# 1. Ouverture, lecture et fermeture du fichier  
# ------------------------------------------------
#
  if append :
    file = open(filename,'a')
  else :
    file = open(filename,'w')
#
# 2. Ecriture du nombre de profils et du type d'obs
#-------------------------------------------------
#
#*SM  nobs = len(date)
  nobs = len(vmr)
  line = "%4i  %i   \n" % (nobs, 1)
  file.write(line)
  file.write(ostype)
#  file.write(ostype+'\n')
#
# 3. Ecriture des profils
#-------------------------------------------------
#
  new_sname = SetSpeciesString(species_name)
#
  for iobs in range(nobs) : 
#
    nbniv = len(press[iobs])

    if isinstance(name,basestring) is True: 
      new_name = SetObsNameString(name)
    else:
      new_name = SetObsNameString(name[iobs])

    if isinstance(extra_code,basestring) is True: 
      new_code = SetObsCodeString(extra_code)
    else:
      new_code = SetObsCodeString(extra_code[iobs])
#
#     Lecture de la 1er ligne de chaque profile
#  
    line = '%s %s %s' % (date[iobs], new_name, new_code)
    line = line + ' %6.2f %7.2f %2.0i\n' % (lat[iobs], lon[iobs], nbniv)    
    file.write(line)
#
#     Lecture des pressions et des concentrations d'ozone
#  
    for iniv in range(nbniv) :
      if not totcol : 
#       try : 
        line  = "%7d %2i\n" % (press[iobs][iniv], 1)
        if len(vmr[iobs]) == 1 :
          line += "[%3s] %12.5e\n" % (new_sname, vmr[iobs][0])
        else : 
          ratio = (obs[iobs][iniv] / vmr[iobs][iniv] - 1. ) * 100.
          line += "[%3s] %12.5e %12.5e %12.5e %8.2f%%  A\n" \
              % (new_sname, vmr[iobs][iniv], obs[iobs][iniv], \
                            obs[iobs][iniv] - vmr[iobs][iniv], ratio)
#       except TypeError:
#         line = "%7d %2i\n%s %10.4e\n" % (press[iobs][iniv], 1, '[ O3]', -1.00000000e+10)
      else : 
        line  = "%7.0d %7.0d %2.0i\n" % (10., 100000., 1)
        line += "[%3s] %12.5e\n" % (new_sname, vmr[iobs])
      file.write(line)
    file.flush()
#
# 4. Retour des variables calculees
# ------------------------------------------------
#
  file.close()
